@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2006, 2007 OpenedHand Ltd.
+ * Copyright (C) 2006, 2007, 2008 OpenedHand Ltd.
  *
  * Author: Jorn Baayen <jorn@openedhand.com>
  *
@@ -31,10 +31,12 @@
 #include <config.h>
 #include <string.h>
 #include <stdlib.h>
+#include <time.h>
+
+#include <libsoup/soup.h>
 
 #include "gssdp-resource-group.h"
 #include "gssdp-resource-browser.h"
-#include "gssdp-error.h"
 #include "gssdp-client-private.h"
 #include "gssdp-protocol.h"
 
@@ -79,6 +81,7 @@ typedef struct {
 
 typedef struct {
         char     *dest_ip;
+        gushort   dest_port;
         char     *target;
         Resource *resource;
 
@@ -94,8 +97,9 @@ resource_group_timeout          (gpointer            user_data);
 static void
 message_received_cb             (GSSDPClient        *client,
                                  const char         *from_ip,
+                                 gushort             from_port,
                                  _GSSDPMessageType   type,
-                                 GHashTable         *headers,
+                                 SoupMessageHeaders *headers,
                                  gpointer            user_data);
 static void
 resource_alive                  (Resource           *resource);
@@ -187,6 +191,18 @@ gssdp_resource_group_dispose (GObject *object)
 
         resource_group = GSSDP_RESOURCE_GROUP (object);
 
+        while (resource_group->priv->resources) {
+                resource_free (resource_group->priv->resources->data);
+                resource_group->priv->resources =
+                        g_list_delete_link (resource_group->priv->resources,
+                                            resource_group->priv->resources);
+        }
+
+        if (resource_group->priv->timeout_id > 0) {
+                g_source_remove (resource_group->priv->timeout_id);
+                resource_group->priv->timeout_id = 0;
+        }
+
         if (resource_group->priv->client) {
                 if (g_signal_handler_is_connected
                         (resource_group->priv->client,
@@ -199,18 +215,6 @@ gssdp_resource_group_dispose (GObject *object)
                 g_object_unref (resource_group->priv->client);
                 resource_group->priv->client = NULL;
         }
-        
-        while (resource_group->priv->resources) {
-                resource_free (resource_group->priv->resources->data);
-                resource_group->priv->resources =
-                        g_list_delete_link (resource_group->priv->resources,
-                                            resource_group->priv->resources);
-        }
-
-        if (resource_group->priv->timeout_id > 0) {
-                g_source_remove (resource_group->priv->timeout_id);
-                resource_group->priv->timeout_id = 0;
-        }
 }
 
 static void
@@ -284,7 +288,6 @@ gssdp_resource_group_class_init (GSSDPResourceGroupClass *klass)
 /**
  * gssdp_resource_group_new
  * @client: The #GSSDPClient to associate with
- * @error: A location to return an error of type #GSSDP_ERROR_QUARK
  *
  * Return value: A new #GSSDPResourceGroup object.
  **/
@@ -392,9 +395,9 @@ gssdp_resource_group_set_available (GSSDPResourceGroup *resource_group,
         if (available) {
                 /* Add re-announcement timer */
                 resource_group->priv->timeout_id =
-                        g_timeout_add (resource_group->priv->max_age * 1000,
-                                       resource_group_timeout,
-                                       resource_group);
+                        g_timeout_add_seconds (resource_group->priv->max_age,
+                                               resource_group_timeout,
+                                               resource_group);
 
                 /* Announce all resources */
                 for (l = resource_group->priv->resources; l; l = l->next)
@@ -576,15 +579,15 @@ resource_group_timeout (gpointer user_data)
  * Received a message
  **/
 static void
-message_received_cb (GSSDPClient      *client,
-                     const char       *from_ip,
-                     _GSSDPMessageType type,
-                     GHashTable       *headers,
-                     gpointer          user_data)
+message_received_cb (GSSDPClient        *client,
+                     const char         *from_ip,
+                     gushort             from_port,
+                     _GSSDPMessageType   type,
+                     SoupMessageHeaders *headers,
+                     gpointer            user_data)
 {
         GSSDPResourceGroup *resource_group;
-        GSList *list;
-        const char *target;
+        const char *target, *mx_str;
         gboolean want_all;
         int mx;
         GList *l;
@@ -600,22 +603,20 @@ message_received_cb (GSSDPClient      *client,
                 return;
 
         /* Extract target */
-        list = g_hash_table_lookup (headers, "ST");
-        if (!list) {
+        target = soup_message_headers_get (headers, "ST");
+        if (!target) {
                 g_warning ("Discovery request did not have an ST header");
 
                 return;
         }
 
-        target = list->data;
-
         /* Is this the "ssdp:all" target? */
         want_all = (strcmp (target, GSSDP_ALL_RESOURCES) == 0);
 
         /* Extract MX */
-        list = g_hash_table_lookup (headers, "MX");
-        if (list)
-                mx = atoi (list->data);
+        mx_str = soup_message_headers_get (headers, "MX");
+        if (mx_str)
+                mx = atoi (mx_str);
         else
                 mx = SSDP_DEFAULT_MX;
 
@@ -636,9 +637,10 @@ message_received_cb (GSSDPClient      *client,
                         /* Prepare response */
                         response = g_slice_new (DiscoveryResponse);
                         
-                        response->dest_ip  = g_strdup (from_ip);
-                        response->target   = g_strdup (target);
-                        response->resource = resource;
+                        response->dest_ip   = g_strdup (from_ip);
+                        response->dest_port = from_port;
+                        response->target    = g_strdup (target);
+                        response->resource  = resource;
 
                         /* Add timeout */
                         response->timeout_id =
@@ -686,7 +688,8 @@ discovery_response_timeout (gpointer user_data)
 {
         DiscoveryResponse *response;
         GSSDPClient *client;
-        char *al, *message;
+        SoupDate *date;
+        char *al, *date_str, *message;
         guint max_age;
 
         response = user_data;
@@ -698,19 +701,26 @@ discovery_response_timeout (gpointer user_data)
 
         al = construct_al (response->resource);
 
+        date = soup_date_new_from_now (0);
+        date_str = soup_date_to_string (date, SOUP_DATE_HTTP);
+        soup_date_free (date);
+
         message = g_strdup_printf (SSDP_DISCOVERY_RESPONSE,
                                    (char *) response->resource->locations->data,
                                    al ? al : "",
                                    response->resource->usn,
                                    gssdp_client_get_server_id (client),
                                    max_age,
-                                   response->target);
+                                   response->target,
+                                   date_str);
 
         _gssdp_client_send_message (client,
                                     response->dest_ip,
+                                    response->dest_port,
                                     message);
 
         g_free (message);
+        g_free (date_str);
         g_free (al);
 
         discovery_response_free (response);
@@ -762,6 +772,7 @@ resource_alive (Resource *resource)
 
         _gssdp_client_send_message (client,
                                     NULL,
+                                    0,
                                     message);
 
         g_free (message);
@@ -786,6 +797,7 @@ resource_byebye (Resource *resource)
         
         _gssdp_client_send_message (client,
                                     NULL,
+                                    0,
                                     message);
 
         g_free (message);
